@@ -18,9 +18,13 @@ Unit tests for the Hyper-V Driver.
 """
 
 import platform
+import sys
 
 import mock
+from nova import exception
+from nova import safe_utils
 from nova.virt import driver as base_driver
+from os_win import exceptions as os_win_exc
 
 from hyperv.nova import driver
 from hyperv.tests.unit import test_base
@@ -28,7 +32,11 @@ from hyperv.tests.unit import test_base
 
 class HyperVDriverTestCase(test_base.HyperVBaseTestCase):
 
-    def setUp(self):
+    FAKE_WIN_2008R2_VERSION = '6.0.0'
+
+    @mock.patch.object(driver.hostops, 'api', mock.MagicMock())
+    @mock.patch.object(driver.HyperVDriver, '_check_minimum_windows_version')
+    def setUp(self, mock_check_minimum_windows_version):
         super(HyperVDriverTestCase, self).setUp()
 
         self.context = 'context'
@@ -42,11 +50,67 @@ class HyperVDriverTestCase(test_base.HyperVBaseTestCase):
         self.driver._rdpconsoleops = mock.MagicMock()
         self.driver._serialconsoleops = mock.MagicMock()
         self.driver._imagecache = mock.MagicMock()
+        self.driver._image_api = mock.MagicMock()
         self.driver._pathutils = mock.MagicMock()
 
+    @mock.patch.object(driver.utilsfactory, 'get_hostutils')
+    def test_check_minimum_windows_version(self, mock_get_hostutils):
+        mock_hostutils = mock_get_hostutils.return_value
+        mock_hostutils.check_min_windows_version.return_value = False
+
+        self.assertRaises(exception.HypervisorTooOld,
+                          self.driver._check_minimum_windows_version)
+
     def test_public_api_signatures(self):
-        self.assertPublicAPISignatures(base_driver.ComputeDriver(None),
-                                       self.driver)
+        # NOTE(claudiub): wrapped functions do not keep the same signature in
+        # Python 2.7, which causes this test to fail. Instead, we should
+        # compare the public API signatures of the unwrapped methods.
+
+        for attr in driver.HyperVDriver.__dict__:
+            class_member = getattr(driver.HyperVDriver, attr)
+            if callable(class_member):
+                mocked_method = mock.patch.object(
+                    driver.HyperVDriver, attr,
+                    safe_utils.get_wrapped_function(class_member))
+                mocked_method.start()
+                self.addCleanup(mocked_method.stop)
+
+        self.assertPublicAPISignatures(base_driver.ComputeDriver,
+                                       driver.HyperVDriver)
+
+    def test_converted_exception(self):
+        self.driver._vmops.get_info.side_effect = (
+            os_win_exc.OSWinException)
+        self.assertRaises(exception.NovaException,
+                          self.driver.get_info, mock.sentinel.instance)
+
+        self.driver._vmops.get_info.side_effect = os_win_exc.HyperVException
+        self.assertRaises(exception.NovaException,
+                          self.driver.get_info, mock.sentinel.instance)
+
+        self.driver._vmops.get_info.side_effect = (
+            os_win_exc.HyperVVMNotFoundException(vm_name='foofoo'))
+        self.assertRaises(exception.InstanceNotFound,
+                          self.driver.get_info, mock.sentinel.instance)
+
+    def test_assert_original_traceback_maintained(self):
+        def bar(self):
+            foo = "foofoo"
+            raise os_win_exc.HyperVVMNotFoundException(vm_name=foo)
+
+        self.driver._vmops.get_info.side_effect = bar
+        try:
+            self.driver.get_info(mock.sentinel.instance)
+            self.fail("Test expected exception, but it was not raised.")
+        except exception.InstanceNotFound:
+            # exception has been raised as expected.
+            _, _, trace = sys.exc_info()
+            while trace.tb_next:
+                # iterate until the original exception source, bar.
+                trace = trace.tb_next
+
+            # original frame will contain the 'foo' variable.
+            self.assertEqual('foofoo', trace.tb_frame.f_locals['foo'])
 
     def test_need_legacy_block_device_info(self):
         self.assertFalse(self.driver.need_legacy_block_device_info)
@@ -79,16 +143,20 @@ class HyperVDriverTestCase(test_base.HyperVBaseTestCase):
         self.driver.list_instances()
         self.driver._vmops.list_instances.assert_called_once_with()
 
-    def test_spawn(self):
+    @mock.patch.object(driver.HyperVDriver, '_recreate_image_meta')
+    def test_spawn(self, mock_recreate_img_meta):
         self.driver.spawn(
             mock.sentinel.context, mock.sentinel.instance,
             mock.sentinel.image_meta, mock.sentinel.injected_files,
             mock.sentinel.admin_password, mock.sentinel.network_info,
             mock.sentinel.block_device_info)
 
+        mock_recreate_img_meta.assert_called_once_with(
+            mock.sentinel.context, mock.sentinel.instance,
+            mock.sentinel.image_meta)
         self.driver._vmops.spawn.assert_called_once_with(
             mock.sentinel.context, mock.sentinel.instance,
-            mock.sentinel.image_meta, mock.sentinel.injected_files,
+            mock_recreate_img_meta.return_value, mock.sentinel.injected_files,
             mock.sentinel.admin_password, mock.sentinel.network_info,
             mock.sentinel.block_device_info)
 
@@ -140,8 +208,7 @@ class HyperVDriverTestCase(test_base.HyperVBaseTestCase):
 
     def test_get_volume_connector(self):
         self.driver.get_volume_connector(mock.sentinel.instance)
-        self.driver._volumeops.get_volume_connector.assert_called_once_with(
-            mock.sentinel.instance)
+        self.driver._volumeops.get_volume_connector.assert_called_once_with()
 
     def test_get_available_resource(self):
         self.driver.get_available_resource(mock.sentinel.nodename)
@@ -358,7 +425,8 @@ class HyperVDriverTestCase(test_base.HyperVBaseTestCase):
             mock.sentinel.network_info, mock.sentinel.block_device_info,
             mock.sentinel.power_on)
 
-    def test_finish_migration(self):
+    @mock.patch.object(driver.HyperVDriver, '_recreate_image_meta')
+    def test_finish_migration(self, mock_recreate_img_meta):
         self.driver.finish_migration(
             mock.sentinel.context, mock.sentinel.migration,
             mock.sentinel.instance, mock.sentinel.disk_info,
@@ -366,10 +434,13 @@ class HyperVDriverTestCase(test_base.HyperVBaseTestCase):
             mock.sentinel.resize_instance, mock.sentinel.block_device_info,
             mock.sentinel.power_on)
 
+        mock_recreate_img_meta.assert_called_once_with(
+            mock.sentinel.context, mock.sentinel.instance,
+            mock.sentinel.image_meta)
         self.driver._migrationops.finish_migration.assert_called_once_with(
             mock.sentinel.context, mock.sentinel.migration,
             mock.sentinel.instance, mock.sentinel.disk_info,
-            mock.sentinel.network_info, mock.sentinel.image_meta,
+            mock.sentinel.network_info, mock_recreate_img_meta.return_value,
             mock.sentinel.resize_instance, mock.sentinel.block_device_info,
             mock.sentinel.power_on)
 
@@ -399,3 +470,42 @@ class HyperVDriverTestCase(test_base.HyperVBaseTestCase):
                                        mock.sentinel.all_instances)
         self.driver._imagecache.update.assert_called_once_with(
             mock.sentinel.context, mock.sentinel.all_instances)
+
+    def _check_recreate_image_meta(self, mock_image_meta, image_ref='',
+                                   instance_img_ref=''):
+        system_meta = {'image_base_image_ref': instance_img_ref}
+        mock_instance = mock.MagicMock(system_metadata=system_meta)
+        self.driver._image_api.get.return_value = {}
+
+        image_meta = self.driver._recreate_image_meta(
+            mock.sentinel.context, mock_instance, mock_image_meta)
+
+        if image_ref:
+            self.driver._image_api.get.assert_called_once_with(
+                mock.sentinel.context, image_ref)
+        else:
+            mock_image_meta.obj_to_primitive.assert_called_once_with()
+            self.assertEqual({'base_image_ref': image_ref},
+                              image_meta['properties'])
+
+        self.assertEqual(image_ref, image_meta['id'])
+
+    def test_recreate_image_meta_has_id(self):
+        mock_image_meta = mock.MagicMock(id=mock.sentinel.image_meta_id)
+        self._check_recreate_image_meta(
+            mock_image_meta, mock.sentinel.image_meta_id)
+
+    def test_recreate_image_meta_instance(self):
+        mock_image_meta = mock.MagicMock()
+        mock_image_meta.obj_attr_is_set.return_value = False
+        self._check_recreate_image_meta(
+            mock_image_meta, mock.sentinel.instance_img_ref,
+            mock.sentinel.instance_img_ref)
+
+    def test_recreate_image_meta_boot_from_volume(self):
+        mock_image_meta = mock.MagicMock()
+        mock_image_meta.obj_attr_is_set.return_value = False
+        mock_image_meta.obj_to_primitive.return_value = {
+            'nova_object.data': {}}
+
+        self._check_recreate_image_meta(mock_image_meta)
